use std::{
    collections::{BTreeMap, HashMap},
    fs,
};

use actix_multipart::form::{tempfile::TempFile, text::Text, MultipartForm};
use actix_web::{post, web, HttpResponse};
use color_eyre::{eyre::eyre, Result};
use lettre::{
    message::{header::ContentType, Attachment, MultiPart, SinglePart},
    Message,
};
use maud::{html, Markup, PreEscaped, DOCTYPE};
use reqwest::Client;
use serde_json::json;
use sqlx::SqliteConnection;
use tracing::{error, info};

#[derive(Debug, MultipartForm)]
struct ContactForm {
    name: Text<String>,
    email: Text<String>,
    staff: Text<String>,
    message: Text<String>,
}

impl From<&ContactForm> for HashMap<i32, String> {
    fn from(form: &ContactForm) -> Self {
        let mut map = HashMap::new();
        map.insert(169, form.name.0.clone());
        map.insert(169, form.email.0.clone());
        map.insert(169, form.staff.0.clone());
        map.insert(169, form.message.0.clone());
        map
    }
}

impl ContactForm {
    async fn build_email(&self) -> Markup {
        html! {
            (DOCTYPE)
                meta charset="utf-8";
                html {
                    head {
                        title { (self.name.0) " filled out a contact form!" }
                        style {
                            "table { border-collapse: collapse; width: 100% }"
                                "td, th { padding: 8px }"
                                "td { text-align: left; width: 70%; word-wrap: break-word }"
                                "th { text-align: right; border-right: 1px solid #ddd }"
                                "tr { border-bottom: 1px solid #ddd }"
                                "h1 { text-align: center }"
                        }
                    }
                    body {
                        h1 { "Contact form for " (self.name.0) "!" }
                        hr;
                        p { "Email: " (self.email.0) }
                        p { "To: " (self.staff.0) }
                        hr;
                        p { (self.message.0) }
                    }
                }
        }
    }

    async fn store_form(&self) -> Result<()> {
        let client = Client::new();
        let map = HashMap::from(self);
        let mut json = HashMap::new();
        json.insert("data", map);

        let res = client
            .post("https://staff.tfcconnection.org/ocs/v2.php/apps/tables/api/2/tables/140/rows")
            .basic_auth("chris", Some("2VHeGxeC^Zf9KqFK^G@Pt!zu2q^6@b"))
            .header("OCS-APIRequest", "true")
            .header("Content-Type", "application/json")
            .json(&json)
            .send()
            .await?;
        if res.status().is_success() {
            let res = res.text().await.unwrap();
            Ok(())
        } else {
            Err(eyre!(
                "Problem in storing data: {:?}",
                res.error_for_status()
            ))
        }
    }

    async fn send_email(&mut self) -> Result<()> {
        let name = self.name.clone();
        let email_subject = format!("Contact form for {}!", name);
        info!("{name} contact form!");
        let email = self.build_email().await;
        let email = SinglePart::html(email.into_string());

        if let Ok(m) = Message::builder()
            .from(
                "TFC ADMIN <no-reply@mail.tfcconnection.org>"
                    .parse()
                    .unwrap(),
            )
            .to("Chris Cochrun <chris@tfcconnection.org>".parse().unwrap())
            .to("Ethan Rose <ethan@tfcconnection.org>".parse().unwrap())
            .subject(email_subject)
            .singlepart(email)
        {
            crate::email::send_email(m).await
        } else {
            Err(eyre!("Email incorrect"))
        }
    }
}

#[post("/api/contact-form")]
pub async fn contact_form(MultipartForm(mut form): MultipartForm<ContactForm>) -> HttpResponse {
    // match form.store_form().await {
    //     Ok(_) => info!("Successfully sent form to nextcloud!"),
    //     Err(e) => error!("There was an erroring sending form to nextcloud: {e}"),
    // }
    match form.send_email().await {
        Ok(_) => info!("Successfully sent email"),
        Err(e) => error!("There was an error sending the email: {e}"),
    }
    HttpResponse::Ok().body("Thank you! We will get back with you shortly!")
}

#[cfg(test)]
mod test {
    use actix_web::test;
    use pretty_assertions::assert_eq;
    use sqlx::Connection;
    use tracing::debug;

    use super::*;

    fn form() -> ContactForm {
        ContactForm {
            name: Text(String::from("Bilbo Braggins")),
            email: Text(String::from("biblo@hobbits.us")),
            staff: Text(String::from("Uncle")),
            message: Text(String::from("Very")),
        }
    }

    #[test]
    async fn test_nc_post() {
        let form = form();
        assert!(!form.name.is_empty());
        let res = form.store_form().await;
        match res {
            Ok(_) => assert!(true, "passed storing test"),
            Err(e) => assert!(false, "Failed storing test: {e}"),
        }
    }

    #[test]
    async fn test_email() {
        let mut form = form();
        assert!(!form.name.is_empty());
        match form.send_email().await {
            Ok(_) => assert!(true, "passed emailing test"),
            Err(e) => assert!(false, "Failed emailing test: {e}"),
        }
    }
}
